#!/bin/bash

# Some utility functions to be used in the install scripts

# Check if version A is greater than or equal to version B
# Returns 0 if true, 1 if false
# Calling:
# is_version_a_gte_version_b "X.Y" "A.B"
# By virtue of ignoring anything after the second element, it can handle version strings with garbage on the end
# It will allow periods, spaces, or dashes as number separators
is_version_a_gte_version_b() {
	# Return 1 when one of the two parameters isn't a version
	if ! [[ "$1" =~ [0-9]*[.\ -][0-9]*.* ]]; then
		return 1
	fi

	if ! [[ "$2" =~ [0-9]*[.\ -][0-9]*.* ]]; then
		return 1
	fi

	local versionA=($(echo "$1" | tr "." " " | tr "-" " "))
	local versionB=($(echo "$2" | tr "." " " | tr "-" " "))

	if [[ ${versionA[0]} -lt ${versionB[0]} ]]; then
		return 1
	elif [[ ${versionA[0]} -gt ${versionB[0]} ]]; then
		return 0
	else
		if [[ ${versionA[1]} -lt ${versionB[1]} ]]; then
			return 1
		elif [[ ${versionA[1]} -gt ${versionB[1]} ]]; then
			return 0
		else
			return 0
		fi
	fi
}

absolute_value() {
	local number=$1

	if [ "$number" -lt 0 ]; then
		echo "$((-number))"
	else
		echo "$number"
	fi
}

list_potentially_backed_up_files() {
	local path=$1
	# f: regular files, d: directories
	local fileType=$2

	# Some files and folders are intentionally left in place during an upgrade - these are listed below
	# Grep for Directories to exclude
	local grepExcludeDirectories="grep -v \"App_Data/Session\" | grep -v \"App_Data/Toolbox\""
	# Grep for Files to exclude
	local grepExcludeFiles="grep -v \"App_Data/License.xml\" | grep -v \"App_Data/User.xml\" | grep -v \"App_Data/Session.db\""

	find "$path" -type "$fileType" 2>/dev/null | eval "$grepExcludeDirectories $(if [ "$fileType" = f ]; then echo "| $grepExcludeFiles"; fi)"
}

function make_backup() {
	# Make a temporary backup folder
	backup_temp_directory=$(mktemp -d /tmp/screenconnect-XXX)

	# Move into the installation directory, after saving the current directory
	local last_working_directory=$(pwd)
	cd "$installation_directory"

	# Create all folders that are in installation folder in backup folder
	list_potentially_backed_up_files ./ d | xargs -I{} mkdir $backup_temp_directory/{} &>/dev/null

	# Copy files we need to back up always
	mv -f "$installation_directory/web.config" "$backup_temp_directory" &>/dev/null
	mv -f "$installation_directory/App_GlobalResources/Default.resx" "$backup_temp_directory/App_GlobalResources/" &>/dev/null
	mv -f "$installation_directory/App_ClientConfig/Elsinore.ScreenConnect.Client.resx" "$backup_temp_directory/App_ClientConfig/" &>/dev/null
	mv -f "$installation_directory/App_Data/SessionGroup.xml" "$backup_temp_directory/App_Data/SessionGroup.xml" &>/dev/null
	mv -f "$installation_directory/App_Data/Role.xml" "$backup_temp_directory/App_Data/Role.xml" &>/dev/null
	mv -f "$installation_directory/App_Data/SessionEventTrigger.xml" "$backup_temp_directory/App_Data/SessionEventTrigger.xml" &>/dev/null

	# Linux doesn't set creation dates
	# We will use the last-modified time of a reference file (Core.dll) to differentiate installation files vs user-modified files
	local reference_mtime_seconds=$(get_file_mtime_seconds "$backup_mtime_reference_file_path")
	for potentially_backed_up_file in $(list_potentially_backed_up_files ./ f); do
		if [ $(absolute_value "$(($(get_file_mtime_seconds "$potentially_backed_up_file") - reference_mtime_seconds))") -gt $backup_mtime_leeway_seconds ]; then
			# Move all files modified more than a minute before or after the reference to backup directory except excluded
			mv "$potentially_backed_up_file" "$backup_temp_directory/$potentially_backed_up_file" &>/dev/null
		else
			# Remove any other files except excluded
			rm -f "$potentially_backed_up_file"
		fi
	done

	# Remove any remaining empty directories, except excluded directories
	# (this will retain folders the excluded files are in, also)
	list_potentially_backed_up_files ./ d | xargs -I{} rmdir {} &>/dev/null

	# Go back to the old working directory
	cd "$last_working_directory"
}

function copy_installation_files() {
	mkdir -p "$installation_directory" &>/dev/null
	cp -r "$SCREENCONNECT_DIRECTORY/"* "$installation_directory/"
	cp "$mono_executable_file_path" "$installation_directory/App_Runtime/bin/mono"
	chmod 755 "$installation_directory/App_Runtime/bin/"*

	local reference_mtime_seconds=$(get_file_mtime_seconds "$backup_mtime_reference_file_path")
	for potentially_backed_up_file in $(list_potentially_backed_up_files "$installation_directory" f); do
		if [ $(absolute_value "$(($(get_file_mtime_seconds "$potentially_backed_up_file") - reference_mtime_seconds))") -gt $backup_mtime_leeway_seconds ]; then
			touch -r "$backup_mtime_reference_file_path" "$potentially_backed_up_file"
		fi
	done
}

function restore_backup() {
	# Pull files that are always backed up
	mv -f "$backup_temp_directory/web.config" "$installation_directory" &>/dev/null
	mv -f "$backup_temp_directory/App_GlobalResources/Default.resx" "$installation_directory/App_GlobalResources/" &>/dev/null
	mv -f "$backup_temp_directory/App_ClientConfig/Elsinore.ScreenConnect.Client.resx" "$installation_directory/App_ClientConfig/" &>/dev/null
	mv -f "$backup_temp_directory/App_Data/SessionGroup.xml" "$installation_directory/App_Data/SessionGroup.xml" &>/dev/null
	mv -f "$backup_temp_directory/App_Data/Role.xml" "$installation_directory/App_Data/Role.xml" &>/dev/null
	mv -f "$backup_temp_directory/App_Data/SessionEventTrigger.xml" "$installation_directory/App_Data/SessionEventTrigger.xml" &>/dev/null

	# Bring in logo if it's there - a lot of customers modify it
	mv -f "$backup_temp_directory/Images/Logo.png" "$installation_directory/Images/Logo.png" &>/dev/null
	if [ $? -eq 0 ]; then
		# If we bring back the logo, make its modified time 5 minutes future so it gets backed up next upgrade
		update_mtime_to_5min_future "$installation_directory/Images/Logo.png" &>/dev/null
	fi

	# Remove all empty directories
	find "$backup_temp_directory" -depth -type d | xargs -I{} rmdir {} &>/dev/null

	if [ -e "$backup_temp_directory" ]; then
		echo "The upgrade process has detected that some files had been modified since last installation."
		echo "These files were saved for review in: $backup_temp_directory"
	fi
}

get_mono_executable_file_path() {
	find "$1" -name 'mono_*' | sort > /tmp/monos

	while read monoFile; do
		if [ -x "$monoFile" ]; then
			"$monoFile" >/dev/null 2>&1
		else
			cp "$monoFile" /tmp/mono
			chmod +x /tmp/mono
			/tmp/mono >/dev/null 2>&1
		fi

		if [ $? -eq 1 ]; then
			echo $monoFile
			break
		fi
	done < /tmp/monos

	rm /tmp/monos
}

# The next several functions are common utility functions for scripts in the Distributions folder

base_linux_dependency_check() {
	ldconfig -p | grep "libsqlite3\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libsqlite3 (critical): For accessing the session database"
	fi

	ldconfig -p | grep "libz\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libz (critical): For compressing web requests, building installers, and encoding video"
	fi

	ldconfig -p | grep "libavcodec\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libavcodec53 (optional): For transcoding videos when using Extended Auditing"
	else
		# If it's available, test if x264 support was built in
		grep "x264_encoder_encode" `ldconfig -p | grep "libavcodec\." -m 1 | awk '{ print $NF }'` &>/dev/null
		if [ $? -ne 0 ]; then
			problematic_dependencies[${#problematic_dependencies[*]}]="libavcodec53 (optional): For transcoding videos when using Extended Auditing"
		fi
	fi

	ldconfig -p | grep "libswscale\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libswscale2 (optional): For transcoding videos when using Extended Auditing"
	fi

	ldconfig -p | grep "libavutil\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libavutil51 (optional): For transcoding videos when using Extended Auditing"
	fi

	ldconfig -p | grep "libavformat\." &>/dev/null
	if [ $? -ne 0 ]; then
		problematic_dependencies[${#problematic_dependencies[*]}]="libavformat53 (optional): For transcoding videos when using Extended Auditing"
	fi
}

# Gets provided file's mtime
# Calling:
# base_linux_get_file_mtime_seconds $file
base_linux_get_file_mtime_seconds() {
	local file=$1
	stat -c %Y "$file"
}

# Set a file's modified time to 5 minutes in the future
# Calling:
# base_linux_update_mtime_to_5min_future $file
base_linux_update_mtime_to_5min_future() {
	local file=$1
	touch -d "@$(date --date '5min')" "$file" &>/dev/null
}

# Build an action to start ScreenConnect
base_linux_build_start_service_action() {
	installer_actions[${#installer_actions[*]}]="\"$initd_directory/$service_name\" start >/dev/null"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Start $service_name service"
}

# Build an action to stop ScreenConnect, remove the init.d script, and remove any rc.d links
base_linux_build_stop_and_remove_service_action() {
	local servicename=$1
	local filepath="$initd_directory/$1"

	# Stop the service
	installer_actions[${#installer_actions[*]}]="\"$filepath\" stop &>/dev/null"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Stop existing service $servicename if running"

	# Removing rc.d links
	installer_actions[${#installer_actions[*]}]="find -L $rcd_directory/rc?.d -samefile \"$filepath\" 2>/dev/null | xargs -I{} rm {}"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Remove startup links pointing to script at $filepath"

	# Remove script itself
	installer_actions[${#installer_actions[*]}]="rm \"$filepath\" -f"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Remove service start script at $filepath"
}

# Grep an init.d directory for an identifying line pointing to the installation directory
# Assumes $initd_directory and $installation_directory are defined
# Fills service_names array
base_linux_get_existing_service_names() {
	service_names=($(grep -l "\"$installation_directory\"" "$initd_directory/"* 2>/dev/null | xargs -I{} basename {} | tr "\n" " "))
}

# Install to init.d/rc.d in linux
# Assumes variables are global
base_linux_build_startup_actions() {
	if [ "$service_name" == "" ]; then
		echo "Service name is blank, setting to \"screenconnect\"!"
		service_name="screenconnect"
	fi

	# Setup init.d
	# Use ,'s as the sed separator since installation_directory will contain slashes
	INITD_SCRIPT="screenconnect"
	installer_actions[${#installer_actions[*]}]="
		cp \"$INSTALL_DIRECTORY/$INITD_SCRIPT\" \"$INSTALL_DIRECTORY/$INITD_SCRIPT.old\";
		sed -e \"s,screenconnectPath=\\\".*\\\",screenconnectPath=\\\"$installation_directory\\\",\" < \"$INSTALL_DIRECTORY/$INITD_SCRIPT.old\" > \"$INSTALL_DIRECTORY/$INITD_SCRIPT\";
		cp \"$INSTALL_DIRECTORY/$INITD_SCRIPT\" \"$INSTALL_DIRECTORY/$INITD_SCRIPT.old\";
		rm \"$INSTALL_DIRECTORY/$INITD_SCRIPT.old\";
		cp \"$INSTALL_DIRECTORY/$INITD_SCRIPT\" \"$initd_directory/$service_name\";
		chmod 755 \"$initd_directory/$service_name\";
	"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Create service script at $initd_directory/$service_name"

	# Setup rc.d
	# Runlevels 2,3,5 are startup with various options (GUI, network)
	# Runlevel 0 is halt, 6 is reboot
	local rcd_link_name="03$service_name"
	installer_actions[${#installer_actions[*]}]="
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc2.d/S$rcd_link_name\" &>/dev/null;
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc3.d/S$rcd_link_name\" &>/dev/null;
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc4.d/S$rcd_link_name\" &>/dev/null;
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc5.d/S$rcd_link_name\" &>/dev/null;
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc0.d/K$rcd_link_name\" &>/dev/null;
		ln -s \"$initd_directory/$service_name\" \"$rcd_directory/rc6.d/K$rcd_link_name\" &>/dev/null;
	"
	installer_action_descriptions[${#installer_action_descriptions[*]}]="Create startup links in $rcd_directory/rcX.d/ directories"
}

base_linux_launch_web_page() {
	echo "To access your new ScreenConnect installation, open a browser and navigate to:"
	echo "$1"
}
