﻿<%@ Control Language="C#" EnableViewState="true" ClassName="Elsinore.ScreenConnect.StatusPanel" %>
<%@ Import Namespace="System.Net.Sockets" %>
<%@ Import Namespace="System.Net.Security" %>
<%@ Import Namespace="System.Diagnostics" %>
<%@ Import Namespace="System.ServiceModel.Configuration" %>
<script runat="server">

	protected override void OnLoad(EventArgs e)
	{
		base.OnLoad(e);

		foreach (var statusTestPanel in this.GetAllControls().OfType<StatusTestPanel>())
		{
			statusTestPanel.CssClass = "StatusTestPanel";
			statusTestPanel.HeaderStyle.CssClass = "StatusTestPanelHeader";
			statusTestPanel.ContentStyle.CssClass = "StatusTestPanelContent";
			statusTestPanel.ExpandImageUrl = "Images/Expand.png";
			statusTestPanel.CollapseImageUrl = "Images/Collapse.png";
			statusTestPanel.LoadingImageUrl = "Images/ActivityIndicator.gif";

			statusTestPanel.IncompleteSettings.StatusText = Resources.Default.StatusPanel_StatusIncompleteText;

			statusTestPanel.ErrorSettings.HeaderStyle.CssClass = "Error";
			statusTestPanel.ErrorSettings.StatusText = Resources.Default.StatusPanel_StatusErrorText;

			statusTestPanel.PassedSettings.HeaderStyle.CssClass = "Passed";
			statusTestPanel.PassedSettings.StatusText = Resources.Default.StatusPanel_StatusPassedText;

			statusTestPanel.WarningSettings.HeaderStyle.CssClass = "Warning";
			statusTestPanel.WarningSettings.StatusText = Resources.Default.StatusPanel_StatusWarningText;

			statusTestPanel.FailedSettings.HeaderStyle.CssClass = "Failed";
			statusTestPanel.FailedSettings.StatusText = Resources.Default.StatusPanel_StatusFailedText;
		}

		if (this.IsPostBack)
		{
			this.CreateAsyncTasks()
				.Where(t => ((StatusTestPanel)t.State).TestResult == TestResult.Incomplete)
				.ForEach(t => this.Page.RegisterAsyncTask(t));

			this.Page.ExecuteRegisteredAsyncTasks();
		}
	}

	protected override void OnPreRender(EventArgs e)
	{
		base.OnPreRender(e);
		this.timer.Enabled = this.GetAllControls().OfType<StatusTestPanel>().Any(stp => stp.TestResult == TestResult.Incomplete);
		this.DataBind();
	}

	protected override bool OnBubbleEvent(object source, EventArgs e)
	{
		var commandEventArgs = e as CommandEventArgs;

		if (commandEventArgs != null && commandEventArgs.CommandName == "Retry")
		{
			var stp = ((Control)source).FindAncestor<StatusTestPanel>();
			stp.TestResult = TestResult.Incomplete;
			stp.TestData = null;
			return true;
		}
		else
		{
			return base.OnBubbleEvent(source, e);
		}
	}

	IEnumerable<PageAsyncTask> CreateAsyncTasks()
	{
		yield return this.CreateAsyncTask<VersionCheckExecutor>(this.versionCheckPanel);
		yield return this.CreateAsyncTask<WindowsFirewallCheckExecutor>(this.windowsFirewallCheckPanel);
		yield return this.CreateAsyncTask<ExternalAccessibilityCheckExecutor>(this.externalAccessibilityCheckPanel);
		yield return this.CreateAsyncTask<BrowserUrlCheckExecutor>(this.browserUrlCheckPanel);
		yield return this.CreateAsyncTask<WebServerExecutor>(this.webServerCheckPanel);
		yield return this.CreateAsyncTask<RelayCheckExecutor>(this.relayCheckPanel);
		yield return this.CreateAsyncTask<SessionManagerCheckExecutor>(this.sessionManagerCheckPanel);
	}

	PageAsyncTask CreateAsyncTask<T>(StatusTestPanel panel)
		where T : PageAsyncExecutor, new()
	{
		var executor = new T();

		return new PageAsyncTask(
			delegate(object sender, EventArgs e, AsyncCallback cb, object extraData)
			{
				var fallbackResult = new AsyncResult<object> { OriginalCallback = cb, OriginalState = extraData };
				var result = default(IAsyncResult);
				var testData = panel.TestData;

				try
				{
					result = executor.Begin(cb, extraData, ref testData);
				}
				catch (Exception ex)
				{
					fallbackResult.Exception = ex;
				}
				
				panel.TestData = testData;

				if (result != null)
					return result;

				fallbackResult.Complete(true);
				return fallbackResult;
			},
			delegate(IAsyncResult result)
			{
				var testResult = panel.TestResult;
				var testData = panel.TestData;

				try
				{
					executor.End(result, ref testData, ref testResult);
					panel.TestResult = testResult;
				}
				catch (Exception)
				{
					panel.TestResult = TestResult.Error;
				}

				panel.TestData = testData;
			},
			delegate(IAsyncResult result)
			{
				Extensions.Try(() => executor.Abort(result));
				panel.TestResult = TestResult.Error;
			},
			panel,
			true
		);
	}

	abstract class PageAsyncExecutor
	{
		public abstract IAsyncResult Begin(AsyncCallback callback, object state, ref object testData);
		public abstract void End(IAsyncResult result, ref object testData, ref TestResult testResult);
		public abstract void Abort(IAsyncResult result);
	}

	abstract class PageAsyncWebRequestExecutor<T> : PageAsyncExecutor
	{
		WebRequest request;

		public abstract string Url { get; }
		public abstract NameValueCollection GetParameters();
		public abstract TestResult GetTestResult(T testData);

		public override IAsyncResult Begin(AsyncCallback callback, object state, ref object testData)
		{
			var parameters = this.GetParameters();
			this.request = ServerExtensions.CreateWebRequest(this.Url, parameters, 0);
			return this.request.BeginGetResponse(callback, state);
		}

		public override void End(IAsyncResult result, ref object testData, ref TestResult testResult)
		{
			using (var response = this.request.EndGetResponse(result))
			using (var stream = response.GetResponseStream())
				testData = ServerExtensions.DeserializeXml<T>(stream);

			testResult = this.GetTestResult((T)testData);
		}

		public override void Abort(IAsyncResult result)
		{
			this.request.Abort();
		}
	}

	class VersionCheckExecutor : PageAsyncWebRequestExecutor<VersionTestResult>
	{
		public override string Url
		{
			get { return ServerConstants.VersionCheckUrl; }
		}

		public override NameValueCollection GetParameters()
		{
			var parameters = new NameValueCollection();
			parameters.Add("Version", Constants.ProductVersion.ToString());
			parameters.Add("Platform", Environment.OSVersion.Platform.ToString());
			parameters.Add("IsPreRelease", Constants.IsPreRelease.ToString());

			var licenseManager = new LicenseManager();
			licenseManager.LoadLicensesFromDefaultPath();

			foreach (var licenseEnvelope in licenseManager.GetLicenseEnvelopes())
			{
				parameters.Add("LicenseID", licenseEnvelope.Contents.LicenseID);
				parameters.Add("LicenseVersion", licenseEnvelope.Contents.VersionString);
			}

			return parameters;
		}

		public override TestResult GetTestResult(VersionTestResult testData)
		{
			return (testData.NewVersionAvailable ? TestResult.Warning : TestResult.Passed);
		}
	}

	class WindowsFirewallCheckExecutor : PageAsyncExecutor
	{
		public override IAsyncResult Begin(AsyncCallback callback, object state, ref object testData)
		{
			var exeName = Process.GetCurrentProcess().MainModule.FileName;
			var webServerPort = ConfigurationManager.AppSettings.GetUriPort(ServerConstants.WebServerListenUriSettingsKey);
			var relayPort = ConfigurationManager.AppSettings.GetUriPort(ServerConstants.RelayListenUriSettingsKey);

			testData = new WindowsFirewallTestResult
			{
				IsFirewallEnabled = ServerToolkit.Instance.IsWindowsFirewallEnabled(),
				IsWebServerPortAllowed = ServerToolkit.Instance.IsWindowsFirewallTcpPortAllowed(exeName, webServerPort),
				IsRelayPortAllowed = ServerToolkit.Instance.IsWindowsFirewallTcpPortAllowed(exeName, relayPort),
			};

			return null;
		}

		public override void End(IAsyncResult result, ref object testData, ref TestResult testResult)
		{
			if (testData != null)
				testResult = ((WindowsFirewallTestResult)testData).IsWebServerPortAllowed && ((WindowsFirewallTestResult)testData).IsRelayPortAllowed ? TestResult.Passed : TestResult.Failed;
			else
				testResult = TestResult.Error;
		}

		public override void Abort(IAsyncResult result)
		{
			// will never occur
		}
	}

	class ExternalAccessibilityCheckExecutor : PageAsyncWebRequestExecutor<ExternalAccessibilityTestResult>
	{
		public override string Url
		{
			get { return ServerConstants.AccessibilityCheckUrl; }
		}

		public override NameValueCollection GetParameters()
		{
			return ServerExtensions.GetExternalAccessibilityCheckParameters(ConfigurationManager.AppSettings, HttpContext.Current.Request.Url);
		}

		public override TestResult GetTestResult(ExternalAccessibilityTestResult testData)
		{
			return (testData.WebServerErrorMessage == null && testData.RelayErrorMessage == null ? TestResult.Passed : TestResult.Failed);
		}
	}

	class BrowserUrlCheckExecutor : PageAsyncWebRequestExecutor<BrowserUrlTestResult>
	{
		public override string Url
		{
			get { return ServerConstants.BrowserUrlCheckUrl; }
		}

		public override NameValueCollection GetParameters()
		{
			var parameters = new NameValueCollection();
			parameters["Host"] = HttpContext.Current.Request.Url.Host;
			return parameters;
		}

		public override TestResult GetTestResult(BrowserUrlTestResult testData)
		{
			return (testData.IsHostResolvable ? TestResult.Passed : TestResult.Warning);
		}
	}

	class WebServerExecutor : PageAsyncExecutor
	{
		WebRequest request;

		public override IAsyncResult Begin(AsyncCallback callback, object state, ref object testData)
		{
			var webServerUri = ServerExtensions.GetWebServerUri(ConfigurationManager.AppSettings, null, false, false).Uri.AbsoluteUri;
			this.request = ServerExtensions.CreateWebRequest(webServerUri, null, 0);
			ServicePointManager.ServerCertificateValidationCallback = (RemoteCertificateValidationCallback)delegate { return true; };
			return request.BeginGetResponse(callback, state);
		}

		public override void End(IAsyncResult result, ref object testData, ref TestResult testResult)
		{
			try
			{
				testData = this.request.RequestUri.AbsoluteUri;

				using (this.request.EndGetResponse(result))
					testResult = TestResult.Passed;
			}
			catch (WebException)
			{
				testResult = TestResult.Failed;
			}
			finally
			{
				ServicePointManager.ServerCertificateValidationCallback = null;
			}
		}

		public override void Abort(IAsyncResult result)
		{
			this.request.Abort();
		}
	}

	class RelayCheckExecutor : PageAsyncExecutor
	{
		Socket socket;

		public override IAsyncResult Begin(AsyncCallback callback, object state, ref object testData)
		{
			var uri = ConfigurationManager.AppSettings.GetUri(ServerConstants.RelayListenUriSettingsKey);
			var endPoint = ServerExtensions.GetListenEndPoints(uri).First();

			if (endPoint.Address.IsAny())
				endPoint.Address = IPAddress.Loopback;

			testData = uri.Uri;

			this.socket = new Socket(endPoint.Address.AddressFamily, SocketType.Stream, ProtocolType.Tcp);
			return this.socket.BeginConnect(endPoint.Address, endPoint.Port, callback, state);
		}

		public override void End(IAsyncResult result, ref object testData, ref TestResult testResult)
		{
			try
			{
				this.socket.EndConnect(result);
				testResult = TestResult.Passed;
			}
			catch (SocketException)
			{
				testResult = TestResult.Failed;
			}
			finally
			{
				this.socket.ShutdownAndDisposeQuietly();
			}
		}

		public override void Abort(IAsyncResult result)
		{
			this.socket.ShutdownAndDisposeQuietly();
		}
	}

	class SessionManagerCheckExecutor : PageAsyncExecutor
	{
		ISessionManagerChannel sessionManager;

		public override IAsyncResult Begin(AsyncCallback callback, object state, ref object testData)
		{
			this.sessionManager = ServiceChannelPool<ISessionManagerChannel>.Instance.Borrow();
			return sessionManager.BeginWaitForChange(0, 0, 0, callback, state);
		}

		public override void End(IAsyncResult result, ref object testData, ref TestResult testResult)
		{
			try
			{
				testData = WebConfigurationManager.GetSection<ClientSection>().Endpoints.OfType<ChannelEndpointElement>().Where(ep => ep.Contract == typeof(ISessionManagerChannel).FullName).Select(ep => ep.Address.ToString()).FirstOrDefault();
				this.sessionManager.EndWaitForChange(result);
				testResult = TestResult.Passed;
			}
			finally
			{
				this.sessionManager.DisposeQuietly();
			}
		}

		public override void Abort(IAsyncResult result)
		{
			this.sessionManager.DisposeQuietly();
		}
	}

</script>
<asp:Timer runat="server" ID="timer" Interval="1" />
<asp:StatusTestPanel runat="server" ID="versionCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.VersionCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.VersionCheckMessage %>" />
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.YourVersionText %>" />
			<dd>
				<%# ((VersionTestResult)Container.TestData).CheckedVersion %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.LatestVersionText %>" />
			<dd>
				<%# ((VersionTestResult)Container.TestData).LatestVersion %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.LatestEligibleVersionText %>" />
			<dd>
				<%# ((VersionTestResult)Container.TestData).LatestEligibleVersion %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.DownloadLocationText %>" />
			<dd>
				<asp:UriHyperLink runat="server" EnableViewState="false" NavigateUrl="<%# ((VersionTestResult)Container.TestData).DownloadLocation %>" Target="_blank" /></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="windowsFirewallCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.WindowsFirewallCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.WindowsFirewallCheckMessage %>" />
		<p runat="server" visible="<%# Container.TestResult == TestResult.Failed %>">
			<asp:LinkButton runat="server" Text="<%$ Resources:Default, StatusPanel.RetestButtonText %>" CommandName="Retry" EnableViewState="false" /></p>
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.FirewallEnabledText %>" />
			<dd>
				<%# ((WindowsFirewallTestResult)Container.TestData).IsFirewallEnabled %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.WebServerAllowedText %>" />
			<dd>
				<%# ((WindowsFirewallTestResult)Container.TestData).IsWebServerPortAllowed %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.RelayAllowedText %>" />
			<dd>
				<%# ((WindowsFirewallTestResult)Container.TestData).IsRelayPortAllowed %></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="externalAccessibilityCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.ExternalAccessibilityCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.ExternalAccessibilityCheckMessage %>" />
		<p runat="server" visible="<%# Container.TestResult == TestResult.Failed %>">
			<asp:LinkButton runat="server" Text="<%$ Resources:Default, StatusPanel.RetestButtonText %>" CommandName="Retry" EnableViewState="false" /></p>
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.WebServerTestUrlText %>" />
			<dd>
				<asp:UriHyperLink runat="server" EnableViewState="false" NavigateUrl="<%# ((ExternalAccessibilityTestResult)Container.TestData).WebServerUri %>" Target="_blank" /></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.WebServerErrorText %>" />
			<dd>
				<%# ((ExternalAccessibilityTestResult)Container.TestData).WebServerErrorMessage %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.RelayTestUrlText %>" />
			<dd>
				<%# ((ExternalAccessibilityTestResult)Container.TestData).RelayUri %></dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.RelayErrorText %>" />
			<dd>
				<%# ((ExternalAccessibilityTestResult)Container.TestData).RelayErrorMessage %></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="browserUrlCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.BrowserUrlCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.BrowserUrlCheckMessage %>" />
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.BrowsableText %>" />
			<dd>
				<%# ((BrowserUrlTestResult)Container.TestData).IsHostResolvable %></dd>
			</dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.YourUrlText %>" />
			<dd>
				<asp:UriHyperLink runat="server" NavigateUrl="<%# this.Request.Url %>" Target="_blank" />
			</dd>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.PossibleUrlsText %>" />
			<dd>
				<asp:Repeater runat="server" DataSource="<%# ((BrowserUrlTestResult)Container.TestData).AlternateHosts %>" EnableViewState="false">
					<ItemTemplate>
						<p style="margin-bottom: 5px; padding: 0px;">
							<asp:UriHyperLink runat="server" NavigateUrl="<%# (new UriBuilder(this.Request.Url) { Host = (string)Container.DataItem }).Uri %>" Target="_blank" />
						</p>
					</ItemTemplate>
				</asp:Repeater>
			</dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="webServerCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.WebServerCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.WebServerCheckMessage %>" />
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.TestUrlText %>" />
			<dd>
				<asp:UriHyperLink runat="server" EnableViewState="false" NavigateUrl="<%# Container.TestData %>" Target="_blank" /></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="relayCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.RelayCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.RelayCheckMessage %>" />
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.TestUrlText %>" />
			<dd>
				<%# Container.TestData %></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
<asp:StatusTestPanel runat="server" ID="sessionManagerCheckPanel" TitleLabelText="<%$ Resources:Default, StatusPanel.SessionManagerCheckHeading %>">
	<AnyResultTemplate>
		<p runat="server" innerhtml="<%$ Resources:Default, StatusPanel.SessionManagerCheckMessage %>" />
		<dl>
			<dt runat="server" innerhtml="<%$ Resources:Default, StatusPanel.TestUrlText %>" />
			<dd>
				<%# Container.TestData %></dd>
		</dl>
	</AnyResultTemplate>
</asp:StatusTestPanel>
